#include <math.h>

#include "mymodel.h"

#ifndef ABS
#define ABS(x) (((x) < 0)? -(x): (x))
#endif

//--------------------------------------------------------------------
MyModel::MyModel()
{
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
MyModel::~MyModel()
{
 delAllCurves();
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void
MyModel::getBoundBox(double* _xmin, double* _xmax,
                     double* _ymin, double* _ymax)
{
 if( m_curves.size() < 1 )
 {
  *_xmin =  0.0;
  *_xmax = 10.0;
  *_ymin =  0.0;
  *_ymax = 10.0;
  return;
 }

 m_curves[0]->getBoundBox(_xmin, _xmax, _ymin, _ymax);
 for(int i = 1; i < m_curves.size(); i++)
 {
  double xmin_c, xmax_c, ymin_c, ymax_c;
  m_curves[i]->getBoundBox(&xmin_c, &xmax_c, &ymin_c, &ymax_c);
  if(xmin_c < *_xmin)
   *_xmin = xmin_c;
  if(xmax_c > *_xmax)
   *_xmax = xmax_c;
  if(ymin_c < *_ymin)
   *_ymin = ymin_c;
  if(ymax_c > *_ymax)
   *_ymax = ymax_c;
 }
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void 
MyModel::selectPick( double _x, double _y, double _tol, bool _shiftkey )
{
 if( m_curves.size() < 1 )
  return;

 double xC, yC;
 int id_target = -1;
 double dmin = _tol;
 for(int i = 0; i < m_curves.size(); i++)
 {
  // Compute distance between given point and curve and
  // update minimum distance
  xC = _x;
  yC = _y;
  double d = m_curves[i]->closestPoint( &xC, &yC );
  if(d < dmin)
  {
   dmin = d;
   id_target = i;
  }
 }

 // Revert selection of picked curve
 if(id_target > -1)
 {
  if( m_curves[id_target]->isSelected() )
  {
   m_curves[id_target]->setSelected(false);
  } 
  else
  {
   m_curves[id_target]->setSelected(true);
  }
 }

 if( !_shiftkey )
 {
  // If shift key is not pressed, unselect all curves except 
  // the picked one (if there was one selected)
  for(int i = 0; i < m_curves.size(); i++)
  {
   if( i != id_target )
    m_curves[i]->setSelected(false);
  }
 }
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void 
MyModel::selectFence( double _xmin, double _xmax,
                      double _ymin, double _ymax, bool _shiftkey )
{
 if( m_curves.size() < 1 )
  return;

 bool inFence;
 double xmin_c, xmax_c, ymin_c, ymax_c;
 for(int i = 0; i < m_curves.size(); i++)
 {
  m_curves[i]->getBoundBox(&xmin_c, &xmax_c, &ymin_c, &ymax_c);
  if( (xmin_c < _xmin) || (xmax_c > _xmax) ||
      (ymin_c < _ymin) || (ymax_c > _ymax) )
   inFence = false;
  else
   inFence = true;

  if( inFence )
  {
   // Select curve inside fence
   m_curves[i]->setSelected(true);
  }
  else
  {
  // If shift key is not pressed, unselect curve outside fence
   if( !_shiftkey )
    m_curves[i]->setSelected(false);
  }
 }
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void 
MyModel::insertCurve( Curve* _curve )
{
 m_curves.push_back(_curve);
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void 
MyModel::delCurve( int _curveId )
{
 // Remove curve from model and delete it 
 delete m_curves[_curveId];
 m_curves.erase( m_curves.begin() + _curveId );
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void 
MyModel::delAllCurves()
{
 while(m_curves.size() > 0)
 {
  delCurve(m_curves.size() - 1);
 }
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
void 
MyModel::delSelectedCurves()
{
 bool curveRemoved;
 do
 {
  curveRemoved = false;
  for(int i = 0; i < m_curves.size(); i++)
  {
   if( m_curves[i]->isSelected() )
   {
    delCurve(i);
    curveRemoved = true;
    break;
   }
  }
 } while( curveRemoved );
}
//--------------------------------------------------------------------

//--------------------------------------------------------------------
bool 
MyModel::snapToCurve( double* _x, double* _y, double _tol )
{
 if(m_curves.size() < 1)
  return false;

 double xC, yC;
 double xClst = *_x;
 double yClst = *_y;
 int id_target = -1;
 double dmin = _tol;
 double d;

 for(int i = 0; i < m_curves.size(); i++)
 {
  // Check to see whether point is close (within given tolerance)
  // to one of curve vertices. If that is the case, continue to 
  // the next curve.
  xC = m_curves[i]->getXinit();
  yC = m_curves[i]->getYinit();
  if( (ABS(*_x-xC) < _tol) && (ABS(*_y-yC) < _tol) )
  {
   d = sqrt((*_x-xC)*(*_x-xC)+(*_y-yC)*(*_y-yC));
   if(d < dmin)
   {
    xClst = xC;
    yClst = yC;
    dmin = d;
    id_target = i;
   }
   continue;
  }

  xC = m_curves[i]->getXend();
  yC = m_curves[i]->getYend();
  if( (ABS(*_x-xC) < _tol) && (ABS(*_y-yC) < _tol) )
  {
   d = sqrt((*_x-xC)*(*_x-xC)+(*_y-yC)*(*_y-yC));
   if(d < dmin)
   {
    xClst = xC;
    yClst = yC;
    dmin = d;
    id_target = i;
   }
   continue;
  }

  // Compute distance between given point and curve and
  // update minimum distance
  xC = *_x;
  yC = *_y;
  d = m_curves[i]->closestPoint( &xC, &yC );
  if(d < dmin)
  {
   xClst = xC;
   yClst = yC;
   dmin = d;
   id_target = i;
  }
 }

 if( id_target < 0 )
  return false;

 // If found a closest point on a curve, return its 
 // coordinates in the given variables.
 *_x = xClst;
 *_y = yClst;
 return true;
}
//--------------------------------------------------------------------
